"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var layouts_1 = require("./layouts");
var ngx_virtual_keyboard_service_1 = require("./ngx-virtual-keyboard.service");
var key_press_interface_1 = require("./key-press.interface");
var NgxVirtualKeyboardComponent = /** @class */ (function () {
    function NgxVirtualKeyboardComponent(_vk, _ref) {
        this._vk = _vk;
        this._ref = _ref;
    }
    NgxVirtualKeyboardComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.scrollPosition = window.pageYOffset;
        this.layoutSubscription = this._vk.layout$.subscribe(function (layout) {
            _this.layout = layout;
        });
        this.shiftSubscription = this._vk.shift$.subscribe(function (isShift) {
            _this.shift = isShift;
        });
        this.capsLockSubscription = this._vk.capsLock$.subscribe(function (isCapsLock) {
            _this.layout = layouts_1.keyboardCapsLockLayout(_this.layout, isCapsLock);
            _this._ref.detectChanges();
        });
        this.caretPositionSubscription = this._vk.caretPosition$.subscribe(function (caretPosition) {
            _this.caretPosition = caretPosition;
        });
        this.disabledSubscription = this._vk.disabled$.subscribe(function (isDisabled) {
            _this.disabled = isDisabled;
            _this._ref.detectChanges();
        });
        this.specialSubscription = this._vk.special$.subscribe(function (isSpecial) {
            _this.special = isSpecial;
            _this.updateLayout();
        });
        this.checkDisabled();
    };
    NgxVirtualKeyboardComponent.prototype.ngAfterContentInit = function () {
        this._vk.setShift(this.shift);
    };
    NgxVirtualKeyboardComponent.prototype.ngOnDestroy = function () {
        this._vk.reset();
    };
    NgxVirtualKeyboardComponent.prototype.close = function () {
        var _this = this;
        this.location.querySelector('virtual-keyboard').remove();
        var body = document.getElementsByTagName('body')[0];
        body.style.paddingBottom = '0px';
        this._vk.reset();
        this.layoutSubscription.unsubscribe();
        this.shiftSubscription.unsubscribe();
        this.capsLockSubscription.unsubscribe();
        this.caretPositionSubscription.unsubscribe();
        this.disabledSubscription.unsubscribe();
        this.specialSubscription.unsubscribe();
        setTimeout(function () {
            window.scrollTo(0, _this.scrollPosition);
        });
    };
    NgxVirtualKeyboardComponent.prototype.getColSize = function () {
        if (this.type === 'numeric' || this.type === 'phone') {
            return 'col-6 m-auto';
        }
        return;
    };
    NgxVirtualKeyboardComponent.prototype.keyPress = function (event) {
        this.caretPosition = this.inputElement.nativeElement.selectionStart;
        if (event.special) {
            this.handleSpecialKey(event);
        }
        else {
            this.handleNormalKey(event.keyValue);
            this.dispatchEvents(event);
            // Toggle shift if it's activated
            if (this.shift) {
                this._vk.toggleShift();
            }
        }
        this.inputElement.nativeElement.focus();
        this.checkDisabled();
    };
    NgxVirtualKeyboardComponent.prototype.handleSpecialKey = function (event) {
        var _this = this;
        switch (event.keyValue) {
            case 'Enter':
                setTimeout(function () {
                    _this.close();
                    _this.inputElement.nativeElement.blur();
                });
                break;
            case 'Escape':
                setTimeout(function () {
                    _this.close();
                });
                break;
            case 'Left':
                this.cursorPosition(true);
                break;
            case 'Right':
                this.cursorPosition(false);
                break;
            case 'Backspace':
                var currentValue = this.inputElement.nativeElement.value;
                var size = currentValue.length;
                if (this.caretPosition > 0) {
                    var start = currentValue.slice(0, this.caretPosition - 1);
                    var end = currentValue.slice(this.caretPosition);
                    this._vk.setCaretPosition(this.caretPosition - 1);
                    this.inputElement.nativeElement.value = "" + start + end;
                    this.inputElement.nativeElement.selectionStart = this.caretPosition;
                    this.inputElement.nativeElement.selectionEnd = this.caretPosition;
                }
                break;
            case 'CapsLock':
                this._vk.toggleCapsLock();
                break;
            case 'Shift':
                this._vk.toggleShift();
                break;
            case 'Special':
            case 'SpecialBack':
                this._vk.toggleSpecial();
                break;
            case 'SpaceBar':
                this.handleNormalKey(' ');
                break;
        }
        var eventInput = new Event('input');
        this.inputElement.nativeElement.dispatchEvent(eventInput);
    };
    NgxVirtualKeyboardComponent.prototype.dispatchEvents = function (event) {
        var eventInit = {
            bubbles: true,
            cancelable: true,
            shiftKey: this.shift,
            key: event.keyValue,
            code: "Key" + event.keyValue.toUpperCase() + "}",
            location: 0
        };
        this.inputElement.nativeElement.dispatchEvent(new KeyboardEvent('keydown', eventInit));
        this.inputElement.nativeElement.dispatchEvent(new KeyboardEvent('keypress', eventInit));
        this.inputElement.nativeElement.dispatchEvent(new Event('input', { bubbles: true }));
        this.inputElement.nativeElement.dispatchEvent(new KeyboardEvent('keyup', eventInit));
    };
    NgxVirtualKeyboardComponent.prototype.checkDisabled = function () {
        var maxLength = this.inputElement.nativeElement.maxLength;
        var valueLength = this.inputElement.nativeElement.value.length;
        this.disabled = maxLength > 0 && valueLength >= maxLength;
    };
    NgxVirtualKeyboardComponent.prototype.handleNormalKey = function (keyValue) {
        var value = '';
        value = [
            this.inputElement.nativeElement.value.slice(0, this.caretPosition),
            keyValue,
            this.inputElement.nativeElement.value.slice(this.caretPosition)
        ].join('');
        this.caretPosition++;
        this.inputElement.nativeElement.value = value;
        this.inputElement.nativeElement.selectionStart = this.caretPosition;
        this.inputElement.nativeElement.selectionEnd = this.caretPosition;
    };
    NgxVirtualKeyboardComponent.prototype.cursorPosition = function (isLeft) {
        this.caretPosition = this.inputElement.nativeElement.selectionStart;
        var size = this.inputElement.nativeElement.value.length;
        if (isLeft && this.caretPosition > 0) {
            this.caretPosition--;
        }
        else if (!isLeft && this.caretPosition !== size) {
            this.caretPosition++;
        }
        this._vk.setCaretPosition(this.caretPosition);
        this.inputElement.nativeElement.selectionStart = this.caretPosition;
        this.inputElement.nativeElement.selectionEnd = this.caretPosition;
    };
    NgxVirtualKeyboardComponent.prototype.updateLayout = function () {
        if (this.special && (this.type === 'extended' || typeof this.type === 'object')) {
            this._vk.setLayout(layouts_1.specialKeyboard);
        }
        else {
            this._vk.setLayout(this.temp);
        }
        this._ref.detectChanges();
    };
    NgxVirtualKeyboardComponent.prototype.onClick = function (e) {
        var vk = document.getElementsByClassName('ngx-virtual-keyboard')[0];
        var pos = e.pageY - window.pageYOffset;
        if (vk && pos < vk.offsetTop && e.target.tagName !== 'INPUT') {
            this.close();
        }
    };
    return NgxVirtualKeyboardComponent;
}());
exports.NgxVirtualKeyboardComponent = NgxVirtualKeyboardComponent;
//# sourceMappingURL=ngx-virtual-keyboard.component.js.map