"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var ngx_virtual_keyboard_component_1 = require("./ngx-virtual-keyboard.component");
var ngx_virtual_keyboard_service_1 = require("./ngx-virtual-keyboard.service");
var layouts_1 = require("./layouts");
var NgxVirtualKeyboardDirective = /** @class */ (function () {
    function NgxVirtualKeyboardDirective(_elRef, _appRef, _compiler, _injector, _vk, componentFactoryResolver) {
        this._elRef = _elRef;
        this._appRef = _appRef;
        this._compiler = _compiler;
        this._injector = _injector;
        this._vk = _vk;
        this.componentFactoryResolver = componentFactoryResolver;
    }
    NgxVirtualKeyboardDirective.prototype.ngAfterContentInit = function () {
        this.location = this.getRootViewContainerNode();
    };
    NgxVirtualKeyboardDirective.prototype.onClickInput = function () {
        this._vk.setOpen(false);
        this.isMouseEvent = true;
    };
    NgxVirtualKeyboardDirective.prototype.onFocus = function () {
        if (this.isActive && !this.isOpened && this.isMouseEvent) {
            this.isMouseEvent = false;
            this._vk.setOpen(true);
            this.clean();
            this.loadComponent();
            this.openKeyboard();
        }
    };
    NgxVirtualKeyboardDirective.prototype.ngOnDestroy = function () {
        this.clean();
    };
    NgxVirtualKeyboardDirective.prototype.ngOnChanges = function (args) {
        if (args['isDisabled'] && this.componentRef) {
            this._vk.setDisabled(args['isDisabled'].currentValue);
        }
    };
    NgxVirtualKeyboardDirective.prototype.loadComponent = function () {
        var layout = this.getLayout();
        var componentFactory = this._compiler.resolveComponentFactory(ngx_virtual_keyboard_component_1.NgxVirtualKeyboardComponent);
        this.componentRef = componentFactory.create(this._injector);
        this.componentRef.instance.layout = layout;
        this.componentRef.instance.temp = layout;
        this.componentRef.instance.inputElement = this._elRef;
        this.componentRef.instance.location = this.location;
        this.componentRef.instance.type = this.type;
        this.componentRef.changeDetectorRef.detectChanges();
        this.componentRootNode = this.getComponentRootNode(this.componentRef);
    };
    NgxVirtualKeyboardDirective.prototype.getRootViewContainer = function () {
        if (this.componentRef) {
            return this.componentRef;
        }
        var rootComponents = this._appRef.components;
        if (rootComponents.length) {
            return rootComponents[0];
        }
        throw new Error('View Container not found! ngUpgrade needs to manually set this via setRootViewContainer.');
    };
    NgxVirtualKeyboardDirective.prototype.getRootViewContainerNode = function () {
        return this.getComponentRootNode(this.getRootViewContainer());
    };
    NgxVirtualKeyboardDirective.prototype.getComponentRootNode = function (componentRef) {
        return componentRef.hostView.rootNodes[0];
    };
    NgxVirtualKeyboardDirective.prototype.openKeyboard = function () {
        this.location.appendChild(this.componentRootNode);
        this.scrollTo();
    };
    NgxVirtualKeyboardDirective.prototype.getLayout = function () {
        var layout;
        this.type = this.layout;
        switch (this.layout) {
            case 'alphanumeric':
                layout = layouts_1.alphanumericKeyboard;
                break;
            case 'azerty':
                layout = layouts_1.azertyKeyboard;
                break;
            case 'extended':
                layout = layouts_1.extendedKeyboard;
                break;
            case 'numeric':
                layout = layouts_1.numericKeyboard;
                break;
            case 'phone':
                layout = layouts_1.phoneKeyboard;
                break;
            default:
                layout = this.layout;
                break;
        }
        return layout;
    };
    NgxVirtualKeyboardDirective.prototype.clean = function () {
        this._vk.reset();
        if (this.componentRef && this.componentRef.instance.layoutSubscription) {
            this.componentRef.instance.layoutSubscription.unsubscribe();
            this.componentRef.instance.shiftSubscription.unsubscribe();
            this.componentRef.instance.capsLockSubscription.unsubscribe();
            this.componentRef.instance.caretPositionSubscription.unsubscribe();
            this.componentRef.instance.disabledSubscription.unsubscribe();
            this.componentRef.instance.specialSubscription.unsubscribe();
        }
        var body = document.getElementsByTagName('body')[0];
        body.style.paddingBottom = '0px';
        if (this.isActive) {
            this.ngModelChange.emit(this._elRef.nativeElement.value);
        }
        var virtualKeyboard = this.location.querySelector('virtual-keyboard');
        if (virtualKeyboard) {
            virtualKeyboard.remove();
        }
    };
    NgxVirtualKeyboardDirective.prototype.findPos = function (obj) {
        var curtop = 0;
        if (obj.offsetParent) {
            do {
                curtop += obj.offsetTop;
            } while (obj = obj.offsetParent);
            return [curtop];
        }
    };
    NgxVirtualKeyboardDirective.prototype.scrollTo = function () {
        var body = document.getElementsByTagName('body');
        var posElm = this.findPos(this._elRef.nativeElement)[0];
        var paddingBottom = window.innerHeight * .30;
        body[0].style.paddingBottom = paddingBottom + "px";
        window.scroll(0, posElm - paddingBottom);
    };
    return NgxVirtualKeyboardDirective;
}());
exports.NgxVirtualKeyboardDirective = NgxVirtualKeyboardDirective;
//# sourceMappingURL=ngx-virtual-keyboard.directive.js.map