const path = require('path');
const pug = require('pug');
const webpack = require('webpack');
const package = require('./package.json');

const ExtractTextPlugin = require('extract-text-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const OpenBrowserPlugin = require('open-browser-webpack-plugin');
const CompressionPlugin = require('compression-webpack-plugin');

const { CheckerPlugin } = require('awesome-typescript-loader');

const PATH_PUBLIC = path.join(__dirname, 'demo');
const PORT = process.env.port || 8080;
const ENV = process.env.NODE_ENV || 'development';
const VERSION = package.version;

// Common config
const webpackConfig = {};

// Common Meta Data
const metaData = {
  AUTHOR: 'Vincent Tellier <vincent@great-seal.co>',
  DESCRIPTION: '',
  KEYWORDS: ''
};

module.exports = {
  devtool: 'eval',
  entry: [
    path.join(PATH_PUBLIC, 'index.ts'),
    path.join(PATH_PUBLIC, 'index.scss')
  ],
  output: {
    path: path.join(__dirname, 'build'),
    publicPath: '/',
    filename: '[name]-[hash:6].js',
    chunkFilename: '[name]-[hash:6]-chunk.js',
    sourceMapFilename: '[name]-[hash:6].map',
  },
  resolve: {
    extensions: [
      '.ts',
      '.js',
      '.pug',
      '.json',
      '.scss',
      '.css'
    ]
  },
  module: {
    rules: [{
      test: /\.ts$/,
      use: [
        'awesome-typescript-loader'
      ]
    }, {
      test: /\.css$/,
      loader: ['style-loader', 'css-loader', 'sass-loader'],
      exclude: path.join(__dirname, 'node_modules')
    }, {
      test: /\.scss$/,
      use: ['style-loader', 'css-loader', 'sass-loader'],
      include: path.join(__dirname, 'demo'),
      exclude: path.join(__dirname, 'node_modules')
    }, {
      test: /\.pug$/,
      use: ['pug-loader', 'pug-html-loader'],
      exclude: path.join(__dirname, 'node_modules')
    }]
  },
  plugins: [
    new ExtractTextPlugin({ filename: '[name]-[hash:8].style.css', allChunks: true }),
    new OpenBrowserPlugin({
      url: `http://localhost:${PORT}`
    }),
    new CheckerPlugin(),
    new HtmlWebpackPlugin({
      filename: 'index.html',
      inject: true,
      head: true,
      templateContent: () => {
        const templateFile = path.join(PATH_PUBLIC, 'index.pug');
        return pug.compileFile(templateFile)(metaData);
      }
    }),
    new webpack.optimize.CommonsChunkPlugin({
      name: 'vendor-[hash:6]',
      filename: 'vendor-[hash:6].js',
      minChunks(module, count) {
        var context = module.context;
        return context && context.indexOf('node_modules') >= 0;
      }
    }),
    new webpack.DefinePlugin({
      process: {
        env: {
          NODE_ENV: JSON.stringify(ENV)
        }
      }
    })
  ]
};